#include "stdafx.h" 
#include "avisynth.h"


class GraMaMa : public GenericVideoFilter {   

int mode;
int a, b, rad, ra2;
bool binarize;

public:

	GraMaMa(PClip _child, int _mode, int _a, int _b, int _rad, int _ra2, bool _binarize, IScriptEnvironment* env);

	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);

};

GraMaMa::GraMaMa(PClip _child, int _mode, int _a, int _b, int _rad, int _ra2, bool _binarize, IScriptEnvironment* env) : 
	GenericVideoFilter(_child), mode(_mode), a(_a), b(_b), rad(_rad), ra2(_ra2), binarize(_binarize) {

    if (!vi.IsPlanar())
      env->ThrowError("GraMaMa: YV12 data only.");

	if ((mode<1)||(mode>6))
		env->ThrowError("GraMaMa: make sure that: 1<=mode<=4");

	if ((rad<0)||(ra2<0))
		env->ThrowError("GraMaMa: make sure that: rad>=0 and rad2>=0");

}

PVideoFrame __stdcall GraMaMa::GetFrame(int n, IScriptEnvironment* env) {
	
	PVideoFrame src = child->GetFrame(n, env);
	env->MakeWritable(&src);
	unsigned char* srcp = src->GetWritePtr(PLANAR_Y);
	const int src_pitch = src->GetPitch(PLANAR_Y);
	const int src_width = src->GetRowSize(PLANAR_Y);
	const int src_height = src->GetHeight(PLANAR_Y);
	
	float res;
	
	for (int h=0; h < src_height; h++) {
		for (int w=0; w < src_width; w++) {
	
		switch (mode) {
			case 1: // circle gradient
				res = (255*sqrt((w-a)*(w-a)+(h-b)*(h-b)))/float(rad);
				break;
			case 2: // square gradient
				if (abs(h-b)>abs(w-a))
					res = 255*abs(h-b);
				else
					res = 255*abs(w-a);
				res/=rad;
				break;
			case 3: // diamond gradient
				res = 255*(abs(w-a)+abs(h-b));	
				res/=rad;
				break;
			case 4: // horizontal/vertical line gradient
				if (ra2>0) { // vertical
					res = 255*abs(h-b);
					res/=ra2;
				} else { // horizontal (rad>0)
					res = 255*abs(w-a);
					res/=rad;
				}
				break;
			case 5: // ellipse gradient
				res = int( 255*sqrt( ((w-a)*(w-a)/float(rad))/float(rad) + ((h-b)*(h-b)/float(ra2))/float(ra2) ) );
				break;
			case 6: // rectangular gradient
				if (abs(h-b)*rad>abs(w-a)*ra2) {
					res = 255*abs(h-b);
					res/=ra2;
				} else {
					res = 255*abs(w-a);
					res/=rad;
				}
				break;
			}
                
			if (res<0)
				res=0;
			if (res>255)
				res=255;
			if ((res<255)&&binarize==true)
				srcp[w] = 0; // change grey into black when binarize==true
			else
				srcp[w] = int(res);
		}

		srcp = srcp + src_pitch;    
	}

	int imgSizeU = src->GetHeight(PLANAR_U) * src->GetPitch(PLANAR_U);
    if (imgSizeU) {
      memset(src->GetWritePtr(PLANAR_U), 128, imgSizeU);
      memset(src->GetWritePtr(PLANAR_V), 128, imgSizeU);
    }

	return src;

}

AVSValue __cdecl Create_GraMaMa(AVSValue args, void* user_data, IScriptEnvironment* env) {
	int r;
	int mode = args[1].AsInt(1);
	if ((mode==5)||(mode==6)) {
		r = args[5].AsInt((args[0].AsClip()->GetVideoInfo().height)/8);
	} else {
		r = args[5].AsInt(0);
	}

    return new GraMaMa(args[0].AsClip(),
						mode,
						args[2].AsInt((args[0].AsClip()->GetVideoInfo().width)/2),
						args[3].AsInt((args[0].AsClip()->GetVideoInfo().height)/2),
						args[4].AsInt((args[0].AsClip()->GetVideoInfo().height)/4),
						r,
						args[6].AsBool(false),
						env);
}

extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env) {
    env->AddFunction("GraMaMa", "c[mode]i[a]i[b]i[rad]i[rad2]i[binarize]b", Create_GraMaMa, 0); 
    
	return "Gradient Mask Maker";
}

